<?php
/**
 * Plugin Name: Omnistream Connector
 * Description: Integrates Omnistream with WordPress themes
 * Author: Omnistream
 * Version: 0.4
 */

// get_option doesn't return default if original string is blank or has spaces; only works if null
function default_if_empty($value, $default) {
  return (isset($value) && trim($value) !== '') ? $value : $default;
}

define('OMNISTREAM_ROOT_URL', 'https://www.omnistre.am');
function omnistream_root_url() {
  return default_if_empty(get_option('omnistream-root-url'), OMNISTREAM_ROOT_URL);
}

define('OMNISTREAM_TRACKER_URL', 'https://tracker.omnistre.am');
function omnistream_tracker_url() {
  return default_if_empty(get_option('omnistream-tracker-url'), OMNISTREAM_TRACKER_URL);
}

add_action('wp_enqueue_scripts', function() {
  // include .js and .css files with timestamp so old ones aren't cached when version changes
  wp_enqueue_script('omnistream-plugin-js',
                    plugins_url() . '/omnistream-connector/js/main.js',
                    array('jquery'),
                    filemtime(WP_PLUGIN_DIR . '/omnistream-connector/js/main.js'),
                    true);

  wp_enqueue_style('omnistream-plugin-css',
                   plugins_url() . '/omnistream-connector/css/main.css',
                   array(),
                   filemtime(WP_PLUGIN_DIR . '/omnistream-connector/css/main.css'));

  // pass parameters from WP into JS files
  wp_localize_script('omnistream-plugin-js',
                     'omnistreamConnectorVars',
                     array('trackerUrl' => omnistream_tracker_url(),
                           'unitId' => omnistream_get_page_unit_ID(),
                           'rootUrl' => omnistream_root_url(),
                           'userTrackerCookie' => $_COOKIE['user_tracker'],
                           'postGuid' => omnistream_get_post_guid()
                          ));

  // register newsletter widget script for possible use later
  wp_register_script('omnistream-newsletter-js',
                     plugins_url() . '/omnistream-connector/js/newsletter.js',
                     array('jquery'),
                     filemtime(WP_PLUGIN_DIR . '/omnistream-connector/js/newsletter.js'));
});



// set a user tracking cookie

add_action('init', function() {
  if (!isset($_COOKIE['user_tracker'])) {
    $cookie_value = uniqid('', true);
    setcookie('user_tracker', $cookie_value, strtotime('+100 years'));
    $_COOKIE['user_tracker'] = $cookie_value;
  }
});

// add meta to header

add_action('wp_head', function() {
  global $post;

  $description = is_single() ?
  get_post_meta($post->ID, 'OmnistreamMetaDescription', true) :
  get_bloginfo('name') . ' - ' . get_bloginfo('description');

  $output = '<meta name="description" content="' . $description . '" />';
  echo $output;
});

// Exclude expired posts from main query
add_action('pre_get_posts', function($query) {
  if (!is_admin()) {
    if ($query->is_main_query() && !$query->is_admin()) {
      $query->set('meta_query', array(
            'relation' => 'OR',
            array(
              'key' => 'OmnistreamPostExpiry',
              'value' => gmdate("Y-m-d\TH:i:s\Z"),
              'compare' => '>'
            ),
            array(
              'key' => 'OmnistreamPostExpiry',
              'value' => '',
              'compare' => 'NOT EXISTS'
            )
      ));
    }
  }
});

// Add support for shortcode to open link in iframe

add_shortcode('a_in_iframe', function($atts, $content) {
  $a = shortcode_atts(array(
                        'href' => ''
                      ), $atts
  );

  $unit_ID = omnistream_get_page_unit_ID();
  $logo_url = omnistream_root_url() . '/v1/content/units/' . $unit_ID . '/logo/medium';
  $src = $a['href'];
  $dom = "<div class='a_in_iframe'>
            <div class='header'>
              <img src='" . $logo_url . "'>
            </div>
            <iframe src='" . $src . "'></iframe>
          </div>";

  return '<a href="#"  class="desktop-hyperlink" data-featherlight="' . $dom . '">' . $content . '</a>
          <a href="' . $src . '" class="mobile-hyperlink">' . $content . '</a>';
});

// Admin menu for plug-in settings

function omnistream_connector_page() {
?>
  <div class="wrap">
    <h2>Omnistream Connector Settings</h2>
    <form method="post" action="options.php">
<?php
  settings_fields('omnistream-connector-group');
  do_settings_sections('omnistream-connector-group');
?>
      <table class="form-table">
        <tr valign="top">
          <th scope="row">Omnistream Root URL</th>
          <td>
            <input type="url" name="omnistream-root-url" style="width:25em" value="<?php echo esc_attr(get_option('omnistream-root-url')); ?>" />
            <p class="description">
              Enter the full URL for the Omnistream server that is publishing to this WordPress blog if not <?php echo OMNISTREAM_ROOT_URL; ?> (production).
            </p>
          </td>
        </tr>
        <tr valign="top">
          <th scope="row">Omnistream Tracker URL</th>
          <td>
            <input type="url" name="omnistream-tracker-url" style="width:25em" value="<?php echo esc_attr(get_option('omnistream-tracker-url')); ?>" />
            <p class="description">
            Enter the full URL for the Omnistream tracker server used for analytics if not <?php echo OMNISTREAM_TRACKER_URL ?> (production).
            </p>
          </td>
        </tr>
        <tr valign="top">
          <th scope="row">Omnistream Default Unit ID</th>
          <td>
            <input type="number" name="omnistream-default-unit-id" style="width:5em" value="<?php echo esc_attr(get_option('omnistream-default-unit-id')); ?>" />
            <p class="description">
              Enter the numerical Unit ID of the parent unit publishing to this blog.
            </p>
          </td>
        </tr>
      </table>
<?php
  submit_button();
?>
    </form>
  </div>
<?php
}

add_action('admin_menu', function() {
  add_options_page(
    'Omnistream Connector',
    'Omnistream Connector',
    'manage_options',
    'omnistream-connector',
    'omnistream_connector_page'
  );
  add_action('admin_init', function() {

    register_setting('omnistream-connector-group', 'omnistream-root-url', function($url) {
      $url = filter_var($url, FILTER_SANITIZE_URL);
      while(substr($url, -1) == '/') {
        $url = substr($url, 0, -1);
      }
      return $url;
    });

    register_setting('omnistream-connector-group', 'omnistream-tracker-url', function($url) {
      $url = filter_var($url, FILTER_SANITIZE_URL);
      while(substr($url, -1) == '/') {
        $url = substr($url, 0, -1);
      }
      return $url;
    });

    register_setting('omnistream-connector-group', 'omnistream-default-unit-id');
  });
});

// This appears to be required for some hosting providers for some reason that is unclear
if (!function_exists('getallheaders')) {
    /**
     * Get all HTTP header key/values as an associative array for the current request.
     *
     * @return string[string] The HTTP header key/value pairs.
     */
    function getallheaders()
    {
        $headers = array();
        $copy_server = array(
            'CONTENT_TYPE'   => 'Content-Type',
            'CONTENT_LENGTH' => 'Content-Length',
            'CONTENT_MD5'    => 'Content-Md5',
        );
        foreach ($_SERVER as $key => $value) {
            if (substr($key, 0, 5) === 'HTTP_') {
                $key = substr($key, 5);
                if (!isset($copy_server[$key]) || !isset($_SERVER[$key])) {
                    $key = str_replace(' ', '-', ucwords(strtolower(str_replace('_', ' ', $key))));
                    $headers[$key] = $value;
                }
            } elseif (isset($copy_server[$key])) {
                $headers[$copy_server[$key]] = $value;
            }
        }
        if (!isset($headers['Authorization'])) {
            if (isset($_SERVER['REDIRECT_HTTP_AUTHORIZATION'])) {
                $headers['Authorization'] = $_SERVER['REDIRECT_HTTP_AUTHORIZATION'];
            } elseif (isset($_SERVER['PHP_AUTH_USER'])) {
                $basic_pass = isset($_SERVER['PHP_AUTH_PW']) ? $_SERVER['PHP_AUTH_PW'] : '';
                $headers['Authorization'] = 'Basic ' . base64_encode($_SERVER['PHP_AUTH_USER'] . ':' . $basic_pass);
            } elseif (isset($_SERVER['PHP_AUTH_DIGEST'])) {
                $headers['Authorization'] = $_SERVER['PHP_AUTH_DIGEST'];
            }
        }
        return $headers;
    }
}

# get the omnistream organization off of the page, either directly, or by scanning the channels of a post
add_filter('determine_current_user', function($user) {
 $headers = getallheaders();
 if (isset($headers['X-Authorization'])) {
   $basic_header = $headers['X-Authorization'];

   if (! empty($basic_header)) {
     $basic_header_args = preg_split("/Basic/", $basic_header);
     $basic_header_hash = $basic_header_args[1];
     $credentials = preg_split("/:/", base64_decode($basic_header_hash));
     if ($credentials[0]) {
       $_SERVER['PHP_AUTH_USER'] = $credentials[0];
     }
     if ($credentials[1]) {
       $_SERVER['PHP_AUTH_PW']   = $credentials[1];
     }
   }
 }
 return $user;
}, 19);

# get the omnistream organization off of the page, either directly, or by scanning the channels of a post
function get_custom_field($name) {
  global $post;
  $value = get_post_meta($post->ID, $name, true);
  if ($value) {
    return $value;
  }

  $categories = get_the_category();
  foreach ($categories as $category) {
    $pages = get_posts(array(
      'name'        => $category->slug,
      'post_type'   => 'page',
      'post_status' => 'publish',
      'numberposts' => 1
    ));
    if ($pages) {
      $value = get_post_meta($pages[0]->ID, $name, true);
      if ($value) {
        return $value;
      };
    }
  }
  return null;
}

function omnistream_get_page_by_slug($page_slug) {
  global $wpdb;
  $sql = $wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE post_name = %s AND post_type = %s AND post_status = 'publish'", $page_slug, 'page');
  $pageID = $wpdb->get_var($sql);
  if ($pageID) {
    return get_post($pageID);
  } else {
    return null;
  }
}

function omnistream_get_page_unit_ID() {
  global $post;

  if (is_single()) {
    $categories = get_the_category();
    $category = $categories[0];
    $page = omnistream_get_page_by_slug($category->slug);
  }
  else {
    $page = omnistream_get_page_by_slug($post->post_name);
  }

  // get page associated with category and find unit ID if there is one
  while ($page) {
    $unit_ID = get_post_meta($page->ID, 'OmnistreamUnitId', true);
    if ($unit_ID) {
      break;
    }
    if ($page->post_parent != 0) {
      $page = get_post($page->post_parent);
    } else {
      $page = null;
    }
  }

  if(isset($unit_ID) && $unit_ID) {
    return $unit_ID;
  }
  else {
    // we couldn't find a category page with metadata
    return get_option('omnistream-default-unit-id', 1);
  }
}

# get the omnistream page either directly, or by scanning the channels of a post
function get_omnistream_page() {
  global $post;
  if ($post->post_type == 'page') {
    return $post;
  }

  $categories = get_the_category();
  foreach ($categories as $category) {
    $pages = get_posts(array(
      'name'        => $category->slug,
      'post_type'   => 'page',
      'post_status' => 'publish',
      'numberposts' => 1
    ));
    if ($pages) {
      return $pages[0];
    }
  }
  return null;
}

function omnistream_get_post_guid() {
  global $post;
  return $post->guid;
}

add_action('wp_footer', function() {
  global $post;
  $post_guid = $post->guid;
  $page = get_omnistream_page();
  $page_guid = $page->guid;
  echo <<<FOOTER_INCLUDES
    <div id="omnistream_page" data-post-guid="$post_guid" data-page-guid="$page_guid" style="display: none;"></div>

FOOTER_INCLUDES;
  if ($action_bar_text = get_custom_field('action_bar_text')) {
    $action_bar_image  = get_custom_field('action_bar_image');
    $action_bar_url    = get_custom_field('action_bar_url');
    if (isset($page->guid)) {
      echo <<<CALL_TO_ACTION_BAR
        <div class="call_to_action_bar pos_right" id="call_to_action_bar">
          <a href="$action_bar_url" target="_blank" data-event="action-bar">
            <span class="call_to_action_text">$action_bar_text</span>
            <img class="call_to_action_img" id="call_to_action_img" src="$action_bar_image" />
          </a>
        </div>
CALL_TO_ACTION_BAR;
    }
  } elseif ($organization = get_custom_field('OmnistreamChatOrganization')) {
    echo <<<CALL_TO_ACTION_CHAT
      <script async org="$organization" src="https://www.omnistre.am/embed.js" profile="chat"></script>

CALL_TO_ACTION_CHAT;
  } elseif ($organization = get_custom_field('OmnistreamIdeaLabOrganization')) {
    echo <<<CALL_TO_ACTION_IDEA_LAB
      <script async org="$organization" src="http://staging.omnistre.am/embed.js" profile="idea-lab"></script>
CALL_TO_ACTION_IDEA_LAB;
  }
});

add_shortcode('action_button', function($atts, $content = null) {
  $a = shortcode_atts(array(
    'text' => 'text',
    'link' => 'link',
  ), $atts);

  $text = $a['text'];
  $link = $a['link'];
  return "<a href='$link' class='action-button' target='_blank' data-event='action-button'>$text</a>";
});

# place events posted by third parties into the pending state, so they don't immediately appear
add_filter('form_to_post_before_create_post', function($post) {
  if (isset($post['post_category_name']) && $post['post_category_name'] == 'Events') {
    $post['post_status'] = 'pending';
  }
  return $post;
});

// create new "ad" post type
add_action('init', function() {
  $args = array(
    'label' => 'Ads',
    'exclude_from_search' => true,
    'publicly_queryable' => false,
    'show_ui' => true,
    'show_in_nav_menus' => true,
    'show_in_menu' => true,
    'supports' => array('title', 'editor', 'custom-fields'),
    'taxonomies' => array('category'),
    'show_in_rest' => true
  );
  register_post_type('omnistream-ad', $args);
});

// add ability to associate meta tags with attachments
add_post_type_support('attachment', 'custom-fields');

function add_event_custom_post_type() {
  $args = array(
    'label' => 'Events',
    'public' => 'true',
    'show_in_menu' => true,
    'supports' => array('title', 'editor', 'custom-fields'),
    'taxonomies' => array('category'),
    'has_archive' => true,
    // This next setting adds "/events" before the event name in the permalink.
    // That means that you can't use "events" as the slug for the events page.
    'rewrite' => array('slug' => 'events'),
    'show_in_rest' => true
  );
  register_post_type('omnistream-event', $args);
}

add_action('init', 'add_event_custom_post_type');

// flush event rewrite
register_activation_hook(__FILE__, function() {
  add_event_custom_post_type(); // is this necessary?
  flush_rewrite_rules();
});

define('OMNISTREAM_PLUGIN_BASE_URL', dirname(__FILE__));
require_once(OMNISTREAM_PLUGIN_BASE_URL . '/subscribe-widget.php');
require_once(OMNISTREAM_PLUGIN_BASE_URL . '/sidebar-ad-widget.php');
require_once(OMNISTREAM_PLUGIN_BASE_URL . '/events-widget.php');
